@@ -31,7 +31,7 @@
 #endif
 
 #ifndef __ALSA_CONF_H
-#define __ALSA_CONF_H
+#define __ALSA_CONF_H /**< header include loop protection */
 
 #ifdef __cplusplus
 extern "C" {
@@ -52,15 +52,15 @@ extern "C" {
 /** \brief Configuration node type. */
 typedef enum _snd_config_type {
 	/** Integer number. */
-        SND_CONFIG_TYPE_INTEGER,
+	SND_CONFIG_TYPE_INTEGER,
 	/** 64-bit integer number. */
-        SND_CONFIG_TYPE_INTEGER64,
+	SND_CONFIG_TYPE_INTEGER64,
 	/** Real number. */
-        SND_CONFIG_TYPE_REAL,
+	SND_CONFIG_TYPE_REAL,
 	/** Character string. */
-        SND_CONFIG_TYPE_STRING,
-        /** Pointer (runtime only, cannot be saved). */
-        SND_CONFIG_TYPE_POINTER,
+	SND_CONFIG_TYPE_STRING,
+	/** Pointer (runtime only, cannot be saved). */
+	SND_CONFIG_TYPE_POINTER,
 	/** Compound node. */
 	SND_CONFIG_TYPE_COMPOUND = 1024
 } snd_config_type_t;
@@ -108,7 +108,7 @@ void snd_config_unref(snd_config_t *top);
 
 int snd_config_search(snd_config_t *config, const char *key,
 		      snd_config_t **result);
-int snd_config_searchv(snd_config_t *config, 
+int snd_config_searchv(snd_config_t *config,
 		       snd_config_t **result, ...);
 int snd_config_search_definition(snd_config_t *config,
 				 const char *base, const char *key,
@@ -122,7 +122,7 @@ int snd_config_search_definition(snd_config_t *config,
  * \param[in] private_data Handle to the \c private_data node.
  * \return A non-negative value if successful, otherwise a negative error code.
  *
- * Use a function of this type to define a custom expansion 
+ * Use a function of this type to define a custom expansion
  */
 typedef int (*snd_config_expand_fcn_t)(snd_config_t **dst, const char *s, void *private_data);
 
