@@ -33,7 +33,7 @@
 #endif
 
 #ifndef __ALSA_SEQ_H
-#define __ALSA_SEQ_H
+#define __ALSA_SEQ_H /**< header include loop protection */
 
 #ifdef __cplusplus
 extern "C" {
@@ -135,7 +135,7 @@ typedef enum snd_seq_client_type {
 	SND_SEQ_USER_CLIENT     = 1,	/**< user client */
 	SND_SEQ_KERNEL_CLIENT   = 2	/**< kernel client */
 } snd_seq_client_type_t;
-                        
+
 /** client MIDI version */
 enum {
 	SND_SEQ_CLIENT_LEGACY_MIDI = 0,		/**< Legacy client */
