@@ -1,19 +1,19 @@
 /* ladspa.h
 
-   Linux Audio Developer's Simple Plugin API Version 1.0[LGPL].
-   Copyright (C) 2000-2001 Richard W.E. Furse, Paul Barton-Davis,
+   Linux Audio Developer's Simple Plugin API Version 1.1[LGPL].
+   Copyright (C) 2000-2002 Richard W.E. Furse, Paul Barton-Davis,
    Stefan Westerfeld.
-   
+
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License
    as published by the Free Software Foundation; either version 2.1 of
    the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.
-   
+
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
@@ -22,13 +22,17 @@
 #ifndef LADSPA_INCLUDED
 #define LADSPA_INCLUDED
 
+#define LADSPA_VERSION "1.1"
+#define LADSPA_VERSION_MAJOR 1
+#define LADSPA_VERSION_MINOR 1
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 /*****************************************************************************/
 
-/* Overview: 
+/* Overview:
 
    There is a large number of synthesis packages in use or development
    on the Linux platform at this time. This API (`The Linux Audio
@@ -72,14 +76,17 @@ extern "C" {
 /* Fundamental data type passed in and out of plugin. This data type
    is used to communicate audio samples and control values. It is
    assumed that the plugin will work sensibly given any numeric input
-   value although it may have a preferred range (see hints below). */
+   value although it may have a preferred range (see hints below).
+
+   For audio it is generally assumed that 1.0f is the `0dB' reference
+   amplitude and is a `normal' signal level. */
 
 typedef float LADSPA_Data;
 
 /*****************************************************************************/
 
-/* Special Plugin Properties: 
- 
+/* Special Plugin Properties:
+
    Optional features of the plugin type are encapsulated in the
    LADSPA_Properties type. This is assembled by ORing individual
    properties together. */
@@ -115,7 +122,7 @@ typedef int LADSPA_Properties;
    (3) The plugin will not access files, devices, pipes, sockets, IPC
    or any other mechanism that might result in process or thread
    blocking.
-      
+
    (4) The plugin will take an amount of time to execute a run() or
    run_adding() call approximately of form (A+B*SampleCount) where A
    and B depend on the machine and host in use. This amount of time
@@ -130,7 +137,7 @@ typedef int LADSPA_Properties;
 
 /*****************************************************************************/
 
-/* Plugin Ports: 
+/* Plugin Ports:
 
    Plugins have `ports' that are inputs or outputs for audio or
    data. Ports can communicate arrays of LADSPA_Data (for audio
@@ -165,23 +172,23 @@ typedef int LADSPA_PortDescriptor;
 
 /*****************************************************************************/
 
-/* Plugin Port Range Hints: 
+/* Plugin Port Range Hints:
 
    The host may wish to provide a representation of data entering or
    leaving a plugin (e.g. to generate a GUI automatically). To make
    this more meaningful, the plugin should provide `hints' to the host
    describing the usual values taken by the data.
-   
+
    Note that these are only hints. The host may ignore them and the
    plugin must not assume that data supplied to it is meaningful. If
    the plugin receives invalid input data it is expected to continue
    to run without failure and, where possible, produce a sensible
    output (e.g. a high-pass filter given a negative cutoff frequency
    might switch to an all-pass mode).
-    
+
    Hints are meaningful for all input and output ports but hints for
    input control ports are expected to be particularly useful.
-   
+
    More hint information is encapsulated in the
    LADSPA_PortRangeHintDescriptor type which is assembled by ORing
    individual hint types together. Hints may require further
@@ -198,7 +205,7 @@ typedef int LADSPA_PortRangeHintDescriptor;
    bound of the valid range. If LADSPA_HINT_SAMPLE_RATE is also
    specified then the value of LowerBound should be multiplied by the
    sample rate. */
-#define LADSPA_HINT_BOUNDED_BELOW 0x1
+#define LADSPA_HINT_BOUNDED_BELOW   0x1
 
 /* Hint LADSPA_HINT_BOUNDED_ABOVE indicates that the UpperBound field
    of the LADSPA_PortRangeHint should be considered meaningful. The
@@ -206,14 +213,15 @@ typedef int LADSPA_PortRangeHintDescriptor;
    bound of the valid range. If LADSPA_HINT_SAMPLE_RATE is also
    specified then the value of UpperBound should be multiplied by the
    sample rate. */
-#define LADSPA_HINT_BOUNDED_ABOVE 0x2
+#define LADSPA_HINT_BOUNDED_ABOVE   0x2
 
 /* Hint LADSPA_HINT_TOGGLED indicates that the data item should be
    considered a Boolean toggle. Data less than or equal to zero should
    be considered `off' or `false,' and data above zero should be
    considered `on' or `true.' LADSPA_HINT_TOGGLED may not be used in
-   conjunction with any other hint. */
-#define LADSPA_HINT_TOGGLED       0x4
+   conjunction with any other hint except LADSPA_HINT_DEFAULT_0 or
+   LADSPA_HINT_DEFAULT_1. */
+#define LADSPA_HINT_TOGGLED         0x4
 
 /* Hint LADSPA_HINT_SAMPLE_RATE indicates that any bounds specified
    should be interpreted as multiples of the sample rate. For
@@ -221,12 +229,12 @@ typedef int LADSPA_PortRangeHintDescriptor;
    the sample rate) could be requested by this hint in conjunction
    with LowerBound = 0 and UpperBound = 0.5. Hosts that support bounds
    at all must support this hint to retain meaning. */
-#define LADSPA_HINT_SAMPLE_RATE   0x8
+#define LADSPA_HINT_SAMPLE_RATE     0x8
 
 /* Hint LADSPA_HINT_LOGARITHMIC indicates that it is likely that the
    user will find it more intuitive to view values using a logarithmic
    scale. This is particularly useful for frequencies and gains. */
-#define LADSPA_HINT_LOGARITHMIC   0x10
+#define LADSPA_HINT_LOGARITHMIC     0x10
 
 /* Hint LADSPA_HINT_INTEGER indicates that a user interface would
    probably wish to provide a stepped control taking only integer
@@ -234,14 +242,97 @@ typedef int LADSPA_PortRangeHintDescriptor;
    integer range required to avoid floating point rounding errors. For
    instance, the integer set {0,1,2,3} might be described as [-0.1,
    3.1]. */
-#define LADSPA_HINT_INTEGER       0x20
+#define LADSPA_HINT_INTEGER         0x20
 
-#define LADSPA_IS_HINT_BOUNDED_BELOW(x) ((x) & LADSPA_HINT_BOUNDED_BELOW)
-#define LADSPA_IS_HINT_BOUNDED_ABOVE(x) ((x) & LADSPA_HINT_BOUNDED_ABOVE)
-#define LADSPA_IS_HINT_TOGGLED(x)       ((x) & LADSPA_HINT_TOGGLED)
-#define LADSPA_IS_HINT_SAMPLE_RATE(x)   ((x) & LADSPA_HINT_SAMPLE_RATE)
-#define LADSPA_IS_HINT_LOGARITHMIC(x)   ((x) & LADSPA_HINT_LOGARITHMIC)
-#define LADSPA_IS_HINT_INTEGER(x)       ((x) & LADSPA_HINT_INTEGER)
+/* The various LADSPA_HINT_HAS_DEFAULT_* hints indicate a `normal'
+   value for the port that is sensible as a default. For instance,
+   this value is suitable for use as an initial value in a user
+   interface or as a value the host might assign to a control port
+   when the user has not provided one. Defaults are encoded using a
+   mask so only one default may be specified for a port. Some of the
+   hints make use of lower and upper bounds, in which case the
+   relevant bound or bounds must be available and
+   LADSPA_HINT_SAMPLE_RATE must be applied as usual. The resulting
+   default must be rounded if LADSPA_HINT_INTEGER is present. Default
+   values were introduced in LADSPA v1.1. */
+#define LADSPA_HINT_DEFAULT_MASK    0x3C0
+
+/* This default values indicates that no default is provided. */
+#define LADSPA_HINT_DEFAULT_NONE    0x0
+
+/* This default hint indicates that the suggested lower bound for the
+   port should be used. */
+#define LADSPA_HINT_DEFAULT_MINIMUM 0x40
+
+/* This default hint indicates that a low value between the suggested
+   lower and upper bounds should be chosen. For ports with
+   LADSPA_HINT_LOGARITHMIC, this should be exp(log(lower) * 0.75 +
+   log(upper) * 0.25). Otherwise, this should be (lower * 0.75 + upper
+   * 0.25). */
+#define LADSPA_HINT_DEFAULT_LOW     0x80
+
+/* This default hint indicates that a middle value between the
+   suggested lower and upper bounds should be chosen. For ports with
+   LADSPA_HINT_LOGARITHMIC, this should be exp(log(lower) * 0.5 +
+   log(upper) * 0.5). Otherwise, this should be (lower * 0.5 + upper *
+   0.5). */
+#define LADSPA_HINT_DEFAULT_MIDDLE  0xC0
+
+/* This default hint indicates that a high value between the suggested
+   lower and upper bounds should be chosen. For ports with
+   LADSPA_HINT_LOGARITHMIC, this should be exp(log(lower) * 0.25 +
+   log(upper) * 0.75). Otherwise, this should be (lower * 0.25 + upper
+   * 0.75). */
+#define LADSPA_HINT_DEFAULT_HIGH    0x100
+
+/* This default hint indicates that the suggested upper bound for the
+   port should be used. */
+#define LADSPA_HINT_DEFAULT_MAXIMUM 0x140
+
+/* This default hint indicates that the number 0 should be used. Note
+   that this default may be used in conjunction with
+   LADSPA_HINT_TOGGLED. */
+#define LADSPA_HINT_DEFAULT_0       0x200
+
+/* This default hint indicates that the number 1 should be used. Note
+   that this default may be used in conjunction with
+   LADSPA_HINT_TOGGLED. */
+#define LADSPA_HINT_DEFAULT_1       0x240
+
+/* This default hint indicates that the number 100 should be used. */
+#define LADSPA_HINT_DEFAULT_100     0x280
+
+/* This default hint indicates that the Hz frequency of `concert A'
+   should be used. This will be 440 unless the host uses an unusual
+   tuning convention, in which case it may be within a few Hz. */
+#define LADSPA_HINT_DEFAULT_440     0x2C0
+
+#define LADSPA_IS_HINT_BOUNDED_BELOW(x)   ((x) & LADSPA_HINT_BOUNDED_BELOW)
+#define LADSPA_IS_HINT_BOUNDED_ABOVE(x)   ((x) & LADSPA_HINT_BOUNDED_ABOVE)
+#define LADSPA_IS_HINT_TOGGLED(x)         ((x) & LADSPA_HINT_TOGGLED)
+#define LADSPA_IS_HINT_SAMPLE_RATE(x)     ((x) & LADSPA_HINT_SAMPLE_RATE)
+#define LADSPA_IS_HINT_LOGARITHMIC(x)     ((x) & LADSPA_HINT_LOGARITHMIC)
+#define LADSPA_IS_HINT_INTEGER(x)         ((x) & LADSPA_HINT_INTEGER)
+
+#define LADSPA_IS_HINT_HAS_DEFAULT(x)     ((x) & LADSPA_HINT_DEFAULT_MASK)
+#define LADSPA_IS_HINT_DEFAULT_MINIMUM(x) (((x) & LADSPA_HINT_DEFAULT_MASK)   \
+					   == LADSPA_HINT_DEFAULT_MINIMUM)
+#define LADSPA_IS_HINT_DEFAULT_LOW(x)     (((x) & LADSPA_HINT_DEFAULT_MASK)   \
+					   == LADSPA_HINT_DEFAULT_LOW)
+#define LADSPA_IS_HINT_DEFAULT_MIDDLE(x)  (((x) & LADSPA_HINT_DEFAULT_MASK)   \
+					   == LADSPA_HINT_DEFAULT_MIDDLE)
+#define LADSPA_IS_HINT_DEFAULT_HIGH(x)    (((x) & LADSPA_HINT_DEFAULT_MASK)   \
+					   == LADSPA_HINT_DEFAULT_HIGH)
+#define LADSPA_IS_HINT_DEFAULT_MAXIMUM(x) (((x) & LADSPA_HINT_DEFAULT_MASK)   \
+					   == LADSPA_HINT_DEFAULT_MAXIMUM)
+#define LADSPA_IS_HINT_DEFAULT_0(x)       (((x) & LADSPA_HINT_DEFAULT_MASK)   \
+					   == LADSPA_HINT_DEFAULT_0)
+#define LADSPA_IS_HINT_DEFAULT_1(x)       (((x) & LADSPA_HINT_DEFAULT_MASK)   \
+					   == LADSPA_HINT_DEFAULT_1)
+#define LADSPA_IS_HINT_DEFAULT_100(x)     (((x) & LADSPA_HINT_DEFAULT_MASK)   \
+					   == LADSPA_HINT_DEFAULT_100)
+#define LADSPA_IS_HINT_DEFAULT_440(x)     (((x) & LADSPA_HINT_DEFAULT_MASK)   \
+					    == LADSPA_HINT_DEFAULT_440)
 
 typedef struct _LADSPA_PortRangeHint {
 
@@ -262,7 +353,7 @@ typedef struct _LADSPA_PortRangeHint {
 
 /*****************************************************************************/
 
-/* Plugin Handles: 
+/* Plugin Handles:
 
    This plugin handle indicates a particular instance of the plugin
    concerned. It is valid to compare this to NULL (0 for C++) but
@@ -273,13 +364,13 @@ typedef void * LADSPA_Handle;
 
 /*****************************************************************************/
 
-/* Descriptor for a Type of Plugin: 
+/* Descriptor for a Type of Plugin:
 
    This structure is used to describe a plugin type. It provides a
    number of functions to examine the type, instantiate it, link it to
    buffers and workspaces and to run it. */
 
-typedef struct _LADSPA_Descriptor { 
+typedef struct _LADSPA_Descriptor {
 
   /* This numeric identifier indicates the plugin type
      uniquely. Plugin programmers may reserve ranges of IDs from a
@@ -339,12 +430,12 @@ typedef struct _LADSPA_Descriptor {
      instantiation function accepts a sample rate as a parameter. The
      plugin descriptor from which this instantiate function was found
      must also be passed. This function must return NULL if
-     instantiation fails. 
+     instantiation fails.
 
      Note that instance initialisation should generally occur in
      activate() rather than here. */
   LADSPA_Handle (*instantiate)(const struct _LADSPA_Descriptor * Descriptor,
-                               unsigned long                     SampleRate);
+			       unsigned long                     SampleRate);
 
   /* This member is a function pointer that connects a port on an
      instantiated plugin to a memory location at which a block of data
@@ -373,8 +464,8 @@ typedef struct _LADSPA_Descriptor {
      However, overlapped buffers or use of a single buffer for both
      audio and control data may result in unexpected behaviour. */
    void (*connect_port)(LADSPA_Handle Instance,
-                        unsigned long Port,
-                        LADSPA_Data * DataLocation);
+			unsigned long Port,
+			LADSPA_Data * DataLocation);
 
   /* This member is a function pointer that initialises a plugin
      instance and activates it for use. This is separated from
@@ -412,7 +503,7 @@ typedef struct _LADSPA_Descriptor {
      then there are various things that the plugin should not do
      within the run() or run_adding() functions (see above). */
   void (*run)(LADSPA_Handle Instance,
-              unsigned long SampleCount);
+	      unsigned long SampleCount);
 
   /* This method is a function pointer that runs an instance of a
      plugin for a block. This has identical behaviour to run() except
@@ -460,7 +551,7 @@ typedef struct _LADSPA_Descriptor {
   /* Once an instance of a plugin has been finished with it can be
      deleted using the following function. The instance handle passed
      ceases to be valid after this call.
-  
+
      If activate() was called for a plugin instance then a
      corresponding call to deactivate() must be made before cleanup()
      is called. */
@@ -498,7 +589,7 @@ typedef struct _LADSPA_Descriptor {
 const LADSPA_Descriptor * ladspa_descriptor(unsigned long Index);
 
 /* Datatype corresponding to the ladspa_descriptor() function. */
-typedef const LADSPA_Descriptor * 
+typedef const LADSPA_Descriptor *
 (*LADSPA_Descriptor_Function)(unsigned long Index);
 
 /**********************************************************************/
