@@ -15,49 +15,89 @@
   Authors: Jaroslav Kysela <perex@perex.cz>
 */
 
-#include "list.h"
 #include "tplg_local.h"
 
 #define SAVE_ALLOC_SHIFT	(13)	/* 8192 bytes */
+#define PRINT_ALLOC_SHIFT	(10)	/* 1024 bytes */
+#define PRINT_BUF_SIZE_MAX	(1024 * 1024)
+#define NEXT_CHUNK(val, shift)	((((val) >> (shift)) + 1) << (shift))
 
-int tplg_save_printf(char **dst, const char *pfx, const char *fmt, ...)
+void tplg_buf_init(struct tplg_buf *buf)
+{
+	buf->dst = NULL;
+	buf->dst_len = 0;
+	buf->printf_buf = NULL;
+	buf->printf_buf_size = 0;
+}
+
+void tplg_buf_free(struct tplg_buf *buf)
+{
+	free(buf->dst);
+	free(buf->printf_buf);
+}
+
+char *tplg_buf_detach(struct tplg_buf *buf)
+{
+	char *ret = buf->dst;
+	free(buf->printf_buf);
+	return ret;
+}
+
+int tplg_save_printf(struct tplg_buf *dst, const char *pfx, const char *fmt, ...)
 {
 	va_list va;
-	char buf[1024], *s;
+	char *s;
 	size_t n, l, t, pl;
+	int ret = 0;
 
 	if (pfx == NULL)
 		pfx = "";
 
 	va_start(va, fmt);
-	n = vsnprintf(buf, sizeof(buf), fmt, va);
+	n = vsnprintf(dst->printf_buf, dst->printf_buf_size, fmt, va);
 	va_end(va);
 
-	if (n >= sizeof(buf))
-		return -EOVERFLOW;
+	if (n >= PRINT_BUF_SIZE_MAX) {
+		ret = -EOVERFLOW;
+		goto end;
+	}
+
+	if (n >= dst->printf_buf_size) {
+		t = NEXT_CHUNK(n + 1, PRINT_ALLOC_SHIFT);
+		s = realloc(dst->printf_buf, t);
+		if (!s) {
+			ret = -ENOMEM;
+			goto end;
+		}
+		dst->printf_buf = s;
+		dst->printf_buf_size = t;
+		va_start(va, fmt);
+		n = vsnprintf(dst->printf_buf, n + 1, fmt, va);
+		va_end(va);
+	}
 
 	pl = strlen(pfx);
-	l = *dst ? strlen(*dst) : 0;
+	l = dst->dst_len;
 	t = l + pl + n + 1;
 	/* allocate chunks */
-	if (*dst == NULL ||
+	if (dst->dst == NULL ||
 	    (l >> SAVE_ALLOC_SHIFT) != (t >> SAVE_ALLOC_SHIFT)) {
-		s = realloc(*dst, ((t >> SAVE_ALLOC_SHIFT) + 1) <<
-							SAVE_ALLOC_SHIFT);
+		s = realloc(dst->dst, NEXT_CHUNK(t, SAVE_ALLOC_SHIFT));
 		if (s == NULL) {
-			free(*dst);
-			*dst = NULL;
-			return -ENOMEM;
+			ret = -ENOMEM;
+			goto end;
 		}
 	} else {
-		s = *dst;
+		s = dst->dst;
 	}
 
 	if (pl > 0)
 		strcpy(s + l, pfx);
-	strcpy(s + l + pl, buf);
-	*dst = s;
-	return 0;
+	strcpy(s + l + pl, dst->printf_buf);
+	dst->dst = s;
+	dst->dst_len = t - 1;
+end:
+	return ret;
 }
 
 int tplg_nice_value_format(char *dst, size_t dst_size, unsigned int value)
@@ -92,6 +132,8 @@ static int tplg_pprint_integer(snd_config_t *n, char **ret)
 		if (llval < INT_MIN || llval > UINT_MAX)
 			return snd_config_get_ascii(n, ret);
 		lval = llval;
+	} else {
+		lval = 0;
 	}
 	err = tplg_nice_value_format(buf, sizeof(buf), (unsigned int)lval);
 	if (err < 0)
@@ -119,7 +161,6 @@ static snd_config_t *sort_config(const char *id, snd_config_t *src)
 	int index, array, count;
 
 	if (snd_config_get_type(src) != SND_CONFIG_TYPE_COMPOUND) {
-
 		if (snd_config_copy(&dst, src) >= 0)
 			return dst;
 		return NULL;
@@ -131,37 +172,35 @@ static snd_config_t *sort_config(const char *id, snd_config_t *src)
 	if (a == NULL)
 		return NULL;
 	array = snd_config_is_array(src);
-	if (array <= 0) {
-		index = 0;
-		snd_config_for_each(i, next, src) {
-			snd_config_t *s = snd_config_iterator_entry(i);
-			a[index++] = s;
-		}
+	index = 0;
+	snd_config_for_each(i, next, src) {
+		snd_config_t *s = snd_config_iterator_entry(i);
+		a[index++] = s;
+	}
+	if (array <= 0)
 		qsort(a, count, sizeof(a[0]), _compar);
-	}
-	if (snd_config_make_compound(&dst, id, count == 1)) {
-		free(a);
-		return NULL;
-	}
+	if (snd_config_make_compound(&dst, id, count == 1))
+		goto lerr;
 	for (index = 0; index < count; index++) {
 		snd_config_t *s = a[index];
 		const char *id2;
 		if (snd_config_get_id(s, &id2)) {
 			snd_config_delete(dst);
-			free(a);
-			return NULL;
+			goto lerr;
 		}
 		s = sort_config(id2, s);
 		if (s == NULL || snd_config_add(dst, s)) {
 			if (s)
 				snd_config_delete(s);
 			snd_config_delete(dst);
-			free(a);
-			return NULL;
+			goto lerr;
 		}
 	}
 	free(a);
 	return dst;
+lerr:
+	free(a);
+	return NULL;
 }
 
 static int tplg_check_quoted(const unsigned char *p)
@@ -187,9 +226,14 @@ static int tplg_check_quoted(const unsigned char *p)
 	return 0;
 }
 
-static int tplg_save_quoted(char **dst, const char *str)
+static unsigned char nibble(unsigned char b)
+{
+	b &= 0x0f;
+	return b < 10 ? b + '0' : b + 'a';
+}
+
+static int tplg_save_quoted(struct tplg_buf *dst, const char *str)
 {
-	static const char nibble[16] = "0123456789abcdef";
 	unsigned char *p, *d, *t;
 	int c;
 
@@ -231,8 +275,8 @@ static int tplg_save_quoted(char **dst, const char *str)
 			} else {
 				*t++ = '\\';
 				*t++ = 'x';
-				*t++ = nibble[(c >> 4) & 0x0f];
-				*t++ = nibble[(c >> 0) & 0x0f];
+				*t++ = nibble(c >> 4);
+				*t++ = nibble(c >> 0);
 			}
 			break;
 		}
@@ -241,7 +285,7 @@ static int tplg_save_quoted(char **dst, const char *str)
 	return tplg_save_printf(dst, NULL, "'%s'", d);
 }
 
-static int tplg_save_string(char **dst, const char *str, int id)
+static int tplg_save_string(struct tplg_buf *dst, const char *str, int id)
 {
 	const unsigned char *p = (const unsigned char *)str;
 
@@ -257,7 +301,7 @@ static int tplg_save_string(char **dst, const char *str, int id)
 	return tplg_save_printf(dst, NULL, "%s", str);
 }
 
-static int save_config(char **dst, int level, const char *delim, snd_config_t *src)
+static int save_config(struct tplg_buf *dst, int level, const char *delim, snd_config_t *src)
 {
 	snd_config_iterator_t i, next;
 	snd_config_t *s;
@@ -356,9 +400,9 @@ retval:
 				err = tplg_save_quoted(dst, id);
 			} else {
 				err = tplg_save_string(dst, id, 1);
-				if (err < 0)
-					return err;
 			}
+			if (err < 0)
+				return err;
 		} else {
 			delim = "";
 		}
@@ -378,7 +422,8 @@ retval:
 	return 0;
 }
 
-static int tplg_save(snd_tplg_t *tplg, char **dst, int gindex, const char *prefix)
+static int tplg_save(snd_tplg_t *tplg, struct tplg_buf *dst,
+		     int gindex, const char *prefix)
 {
 	struct tplg_table *tptr;
 	struct tplg_elem *elem;
@@ -401,8 +446,9 @@ static int tplg_save(snd_tplg_t *tplg, char **dst, int gindex, const char *prefi
 			if (gindex >= 0 && elem->index != gindex)
 				continue;
 			if (tptr->save == NULL && tptr->gsave == NULL) {
-				SNDERR("unable to create %s block (no callback)",
-				       tptr->id);
+				snd_error(TOPOLOGY, "unable to create %s block (no callback)",
+						    tptr->id);
+
 				err = -ENXIO;
 				goto _err;
 			}
@@ -437,8 +483,9 @@ static int tplg_save(snd_tplg_t *tplg, char **dst, int gindex, const char *prefi
 			}
 			err = tptr->save(tplg, elem, dst, count > 1 ? pfx2 : prefix);
 			if (err < 0) {
-				SNDERR("failed to save %s elements: %s",
-				       tptr->id, snd_strerror(-err));
+				snd_error(TOPOLOGY, "failed to save %s elements: %s",
+						    tptr->id, snd_strerror(-err));
+
 				goto _err;
 			}
 		}
@@ -462,8 +509,6 @@ static int tplg_save(snd_tplg_t *tplg, char **dst, int gindex, const char *prefi
 	return 0;
 
 _err:
-	free(*dst);
-	*dst = NULL;
 	return err;
 }
 
@@ -518,9 +563,9 @@ static int tplg_index_groups(snd_tplg_t *tplg, int **indexes)
 
 int snd_tplg_save(snd_tplg_t *tplg, char **dst, int flags)
 {
+	struct tplg_buf buf, buf2;
 	snd_input_t *in;
 	snd_config_t *top, *top2;
-	char *dst2;
 	int *indexes, *a;
 	int err;
 
@@ -528,37 +573,43 @@ int snd_tplg_save(snd_tplg_t *tplg, char **dst, int flags)
 	assert(dst);
 	*dst = NULL;
 
+	tplg_buf_init(&buf);
+
 	if (flags & SND_TPLG_SAVE_GROUPS) {
 		err = tplg_index_groups(tplg, &indexes);
 		if (err < 0)
 			return err;
 		for (a = indexes; err >= 0 && *a >= 0; a++) {
-			err = tplg_save_printf(dst, NULL,
+			err = tplg_save_printf(&buf, NULL,
 					       "IndexGroup.%d {\n",
 					       *a);
 			if (err >= 0)
-				err = tplg_save(tplg, dst, *a, "\t");
+				err = tplg_save(tplg, &buf, *a, "\t");
 			if (err >= 0)
-				err = tplg_save_printf(dst, NULL, "}\n");
+				err = tplg_save_printf(&buf, NULL, "}\n");
 		}
 		free(indexes);
 	} else {
-		err = tplg_save(tplg, dst, -1, NULL);
+		err = tplg_save(tplg, &buf, -1, NULL);
 	}
 
 	if (err < 0)
 		goto _err;
 
-	if (*dst == NULL)
-		return -EINVAL;
+	if (buf.dst == NULL) {
+		err = -EINVAL;
+		goto _err;
+	}
 
-	if (flags & SND_TPLG_SAVE_NOCHECK)
+	if (flags & SND_TPLG_SAVE_NOCHECK) {
+		*dst = tplg_buf_detach(&buf);
 		return 0;
+	}
 
 	/* always load configuration - check */
-	err = snd_input_buffer_open(&in, *dst, strlen(*dst));
+	err = snd_input_buffer_open(&in, buf.dst, strlen(buf.dst));
 	if (err < 0) {
-		SNDERR("could not create input buffer");
+		snd_error(TOPOLOGY, "could not create input buffer");
 		goto _err;
 	}
 
@@ -571,7 +622,7 @@ int snd_tplg_save(snd_tplg_t *tplg, char **dst, int flags)
 	err = snd_config_load(top, in);
 	snd_input_close(in);
 	if (err < 0) {
-		SNDERR("could not load configuration");
+		snd_error(TOPOLOGY, "could not load configuration");
 		snd_config_delete(top);
 		goto _err;
 	}
@@ -579,7 +630,7 @@ int snd_tplg_save(snd_tplg_t *tplg, char **dst, int flags)
 	if (flags & SND_TPLG_SAVE_SORT) {
 		top2 = sort_config(NULL, top);
 		if (top2 == NULL) {
-			SNDERR("could not sort configuration");
+			snd_error(TOPOLOGY, "could not sort configuration");
 			snd_config_delete(top);
 			err = -EINVAL;
 			goto _err;
@@ -588,20 +639,20 @@ int snd_tplg_save(snd_tplg_t *tplg, char **dst, int flags)
 		top = top2;
 	}
 
-	dst2 = NULL;
-	err = save_config(&dst2, 0, NULL, top);
+	tplg_buf_init(&buf2);
+	err = save_config(&buf2, 0, NULL, top);
 	snd_config_delete(top);
 	if (err < 0) {
-		SNDERR("could not save configuration");
+		snd_error(TOPOLOGY, "could not save configuration");
 		goto _err;
 	}
 
-	free(*dst);
-	*dst = dst2;
+	tplg_buf_free(&buf);
+	*dst = tplg_buf_detach(&buf2);
 	return 0;
 
 _err:
-	free(*dst);
+	tplg_buf_free(&buf);
 	*dst = NULL;
 	return err;
 }
